<?php

namespace Fastrack\Property;

use Fastrack\Location\Place;
use PlatoCreative\Imgix\Imgix;
use Psr\Container\NotFoundExceptionInterface;
use Psr\SimpleCache\CacheInterface;
use Psr\SimpleCache\InvalidArgumentException;
use SilverStripe\CMS\Controllers\ContentController;
use SilverStripe\Control\Director;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Control\HTTPResponse_Exception;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Security;

class ApiController extends ContentController
{

	public const PROPERTIES_LISTING = '/api/properties';

	private const QUERY_PARAM_PROPERTY_ID = 'property-id';

	/**
	 * @config
	 */
	private static string $room_gallery_key = 'gallery';

	/**
	 * @config
	 */
	private static string $property_gallery_key = 'gallery';

	/**
	 * Array of statistics of a property with 'name', 'key' and 'icon'
	 * For example:
	 *  [
	 * 		['name' => 'Villas', 'key' => 'villas', 'icon' => '/images/wbi/property-fill-icon.svg'],
	 * ]
	 *
	 * @config
	 */
	private static array $property_key_feature_settings = [];

	/**
	 * Key room features
	 * Refer to $property_key_feature_settings for specification
	 *
	 * @config
	 */
	private static array $room_key_feature_settings = [];

	/**
	 * Property for feature groups
	 *
	 * name: json key
	 * feature group key: feature group to search for
	 *
	 * for example:
	 * [
	 * 		['name' => 'usps', 'featureGroupKey' => 'unique-selling-points'],
	 * ]
	 *
	 * @config
	 */
	private static array $property_feature_groups = [];

	/**
	 * Room for feature groups.
	 * Refer to $property_feature_groups for array structure
	 *
	 * @config
	 */
	private static array $room_feature_groups = [];

	/**
	 * @config
	 */
	private static int $max_galleryImages = 4;

	private static array $allowed_actions = [
		'places',
		'properties',
		'property'
	];

	private static array $url_handlers = [
		'' => 'properties',
		'$PropertySlug!/places' => 'places',
		'$PropertySlug!' => 'property',
	];

	public function properties(): HTTPResponse
	{
		$CACHE_KEY = 'properties';
		$properties = $this->fetchFromCache($CACHE_KEY, []);
		if (!count($properties)) {
			foreach (Property::get() as $property) {
				$properties[] = $this->buildProperty($property);
			}
			$this->setCache($properties, $CACHE_KEY);
		}
		return $this->constructResponse($properties);
	}


	/**
	 * @throws HTTPResponse_Exception
	 */
	public function property(): HTTPResponse
	{
		$property = $this->getProperty();
		$this->constructResponse($this->buildProperty($property));
		return $this->getResponse();
	}

	/**
	 * @throws HTTPResponse_Exception
	 */
	public function places(): HTTPResponse
	{
		$property = $this->getProperty();
		$places = array_map(function ($place) {
			return $this->buildPlace($place);
		}, $property->Places()->toArray());
		$this->constructResponse($places);
		return $this->getResponse();
	}


	/**
	 * @throws HTTPResponse_Exception
	 */
	private function getProperty(): Property
	{
		$propertySlug = $this->getRequest()->param('PropertySlug');
		$cacheKey = 'property:' . $propertySlug;
		$property = $this->fetchFromCache($cacheKey, null);
		if (is_null($property)) {
			$property = Property::get()->find('Slug', $propertySlug);
			if (is_null($property)) {
				$this->httpError(404);
			}
			$this->setCache($property, $cacheKey);
		}
		return $property;
	}

	private function buildProperty(Property $property): array
	{
		$rooms = [];
		foreach ($property->Rooms() as $room) {
			$rooms[] = $this->buildRoom($room);
		}
		$propertyData = [
			'id' => $property->ID,
			'pmsId' => $property->PmsPropertyId,
			'name' => $property->Name,
			'slug' => $property->Slug,
			'rooms' => $rooms,
			'uniqueSellingPoints' => [],
			'keyFeatures' => $this->getKeyFeatures($property, $this->config()->get('property_key_feature_settings')),
			'fromPrice' => 0,
			'phone' => $property->getField('Phone'),
			'email' => $property->getField('Email'),
			'excerpt' => $property->Excerpt,
			'content' => $property->Content,
			'link' => $property->Link(),
			'slides' => $this->buildGallery($property, $this->config()->get('property_gallery_key')),
			'address' => $this->buildAddress($property),
		];
		$this->addFeatureGroups($propertyData, $property);
		$this->extend('updateBuildProperty', $propertyData, $property);
		return $propertyData;
	}

	private function buildRoom(Room $room): array
	{
		$pmsIds = [];
		foreach (explode(',', trim($room->PmsRoomId)) as $id) {
			$pmsIds[] = intval(trim($id));
		}
		$roomData = [
			'id' => $room->ID,
			'pmsId' => $pmsIds,
			'name' => $room->Name,
			'slug' => $room->Slug,
			'bathrooms' => $room->NumberOfBathrooms,
			'bedrooms' => $room->NumberOfRooms,
			'guests' => $room->NumberOfGuests,
			'order' => $room->SortOrder,
			'excerpt' => $room->Excerpt,
			'content' => $room->Content,
			'image' => $this->buildImage($room->Image()),
			'slides' => $this->buildGallery($room, $this->config()->get('room_gallery_key')),
			'quotes' => [],
			'keyFeatures' => $this->getKeyFeatures($room, $this->config()->get('room_key_feature_settings'))
		];
		$this->addFeatureGroups($roomData, $room);
		$this->extend('updateBuildRoom', $roomData, $room);
		return $roomData;
	}

	private function buildGallery($model, string $galleryKey): array
	{
		$maxImages = intval($this->config()->get('max_galleryImages'));
		$slides = [];
		$gallery = $model->Galleries()->find('Key', $galleryKey);
		if ($gallery) {
			foreach ($gallery->Slides() as $slide) {
				if (count($slides) === $maxImages) {
					break;
				}
				$image = $this->buildImage($slide->Image());
				if ($image) {
					$slides[] = [
						'title' => $slide->Title,
						'image' => $this->buildImage($slide->Image()),
					];
				}

			}
		}
		return $slides;
	}

	private function buildPlace(Place $place): array
	{
		$placeData = [
			'id' => $place->ID,
			'name' => $place->Name,
			'latitude' => floatval($place->Latitude),
			'longitude' => floatval($place->Longitude),
			'address' => $place->getAddressFormatted(),
			'category' => null,
		];
		$category = $place->Category();
		if ($category) {
			$icon = $category->Image;
			$placeData['category'] = [
				'id' => $category->ID,
				'name' => $category->Name,
				'icon' => $icon ? $icon->ScaleMaxWidth(300)->Link() : null,
			];
		}
		$this->extend('updateBuildPlace', $placeData, $place);
		return $placeData;
	}


	private function buildImage(?Imgix $image): ?array
	{
		if (!$image->exists()) {
			return null;
		}
		$scale = $image->useImgix();
		return [
			'alt' => $image->Title,
			'md' => $scale ? $image->ScaleMaxWidth(992)->AbsoluteLink() : $image->AbsoluteLink(),
			'lg' => $scale ? $image->ScaleMaxWidth(1920)->AbsoluteLink() : $image->AbsoluteLink(),
		];
	}

	private function buildAddress(Property $property): array
	{
		return [
			'address' => $property->getAddressFormatted(),
			'latitude' => floatval($property->Latitude),
			'longitude' => floatval($property->Longitude),
		];
	}

	private function constructResponse($data): HTTPResponse
	{
		$this->getResponse()->setBody(json_encode($data));
		$this->getResponse()->addHeader("Content-type", "application/json");
		return $this->getResponse();
	}

	private function getKeyFeatures(DataObject $model, array $settings): array
	{
		$features = [];
		foreach ($settings as $setting) {
			$features[] = [
				'name' => $setting['name'],
				'value' => $model->getField($setting['key']),
				'icon' => $setting['icon'],
			];
		}
		return $features;
	}

	private function addFeatureGroups(array &$data, DataObject $model)
	{
		$keys = (get_class($model) === Property::class) ?
			$this->config()->get('property_feature_groups') :
			$this->config()->get('room_feature_groups');

		foreach ($keys as $key) {
			$data[$key['name']] = $this->buildFeatureGroup($model, $key['featureGroupKey']);
		}
	}

	private function buildFeatureGroup(DataObject $model, $key): array
	{
		$features = [];
		$featureGroup = $model->FeatureGroups()->find('Key', $key);
		if ($featureGroup) {
			foreach ($featureGroup->Features() as $featureItem) {
				$features[] = [
					'name' => $featureItem->getField('Name'),
					'image' => $this->buildImage($featureItem->Image()),
				];
			}
		}
		return $features;
	}

	private function fetchFromCache($key, $default)
	{
		if (!$this->shouldCache()) return $default;
		try {
			return $this->getCache()->get($key, $default);
		} catch (NotFoundExceptionInterface|InvalidArgumentException $e) {
			return $default;
		}
	}

	private function setCache($value, $key) {
		if (!$this->shouldCache()) return;
		try {
			$this->getCache()->set($key, $value, 600);
		} catch (NotFoundExceptionInterface|InvalidArgumentException $ignore) {}
	}

	/**
	 * @throws NotFoundExceptionInterface
	 */
	private function getCache(): CacheInterface
	{
		return Injector::inst()->get(CacheInterface::class . '.propertyApiCache');
	}

	private function shouldCache(): bool
	{
		if (Security::getCurrentUser()) {
			return false;
		}
		if (Director::isDev()) {
			return false;
		}
		return true;
	}


}
