<?php

namespace Fastrack\Property;

use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Config\Config;
use SilverStripe\Core\Environment;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\ValidationException;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;

/**
 * @property string Title
 * @property string Name
 * @property string Key
 */
class FeatureGroup extends DataObject implements PermissionProvider
{
	private static $table_name = 'feature_group';

	private static $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar(255)',
		'Key' => 'Varchar(255)'
	];

	private static $has_one = [
		'Room' => Room::class,
		'Property' => Property::class,
	];

	private static $many_many = [
		'Features' => Feature::class,
	];

	private static $indexes = [
		'Key' => true,
	];

	private static $translate = [
		'Title',
		'Name',
	];

	/**
	 * @config
	 */
	private static array $hidden_fields = [
		'Title',
		'PropertyID',
		'RoomID',
	];

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		ModelUtilities::hideConfigFields(self::class, $fields);
		return $fields;
	}

	public function onBeforeWrite()
	{
		$this->Title = $this->Name;
		parent::onBeforeWrite();
	}

	public function CMSEditLink()
	{
		return Controller::join_links([
			$this->Property()->CMSEditLink(),
			'ItemEditForm',
			'field',
			'FeatureGroups',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink()
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public static function createDefaultFeatureGroups(DataObject $object)
	{
		$class = substr(strrchr($object->getClassName(), '\\'), 1);
		$setting = Environment::getEnv(join('_', ['DEFAULT_FEATURE_GROUPS', $class]));
		if ($setting) {
			$featureGroupKeys = explode(',', $setting);
			foreach ($featureGroupKeys as $featureGroupKey) {
				self::createFeatureGroup($object, $featureGroupKey);
			}
		}
		self::createDefaultFeatureGroupsFromConfig($object);
	}

	private static function createDefaultFeatureGroupsFromConfig(DataObject $dataObject)
	{
		$featureGroupKeys = Config::inst()->get($dataObject->getClassName(), 'default_feature_groups');
		if (!$featureGroupKeys) return;
		foreach ($featureGroupKeys as $featureGroupKey) {
			self::createFeatureGroup($dataObject, $featureGroupKey);
		}
	}

	private static function createFeatureGroup(DataObject $dataObject, string $featureGroupKey)
	{
		if ($dataObject->FeatureGroups()->find('Key', $featureGroupKey)) return;
		try {
			$featureGroup = new FeatureGroup();
			$featureGroup->setField('Name', Slug::deSlug($featureGroupKey));
			$featureGroup->setField('Key', trim($featureGroupKey));
			$featureGroup->write();
			$dataObject->FeatureGroups()->add($featureGroup);
		} catch (ValidationException $ignore) {}
	}

	public function providePermissions()
	{
		$permissions = [
			'MANAGE_FEATURE_GROUPS' => [
				'name' => 'Manage feature groups',
				'category' => 'Features',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		return Permission::check(['MANAGE_FEATURE_GROUPS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_FEATURE_GROUPS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_FEATURE_GROUPS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_FEATURE_GROUPS']);
	}
}
