<?php

namespace Fastrack\Property;

use Ballen\Distical\Calculator;
use Ballen\Distical\Entities\LatLong;
use Fastrack\Gallery\Gallery;
use Fastrack\Location\Place;
use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use PlatoCreative\Imgix\Imgix;
use Psr\Log\LoggerInterface;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Environment;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\HeaderField;
use SilverStripe\ORM\ArrayList;
use SilverStripe\ORM\DataList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;
use UndefinedOffset\SortableGridField\Forms\GridFieldSortableRows;

/**
 * @method HasManyList EventRooms()
 * @method HasManyList FeatureGroups()
 * @method HasManyList Galleries()
 * @method HasManyList Restaurants()
 * @method HasManyList Reviews()
 * @method HasManyList Rooms()
 * @method ManyManyList Places()
 * @method string getAddressFormatted()
 * @property int PmsPropertyId
 * @property string Name
 * @property string Slug
 * @property string $Excerpt
 * @property string $Content
 * @property string $Latitude
 * @property string $Longitude
 */
class Property extends DataObject implements PermissionProvider
{

	private static $table_name = 'property';

	private static $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar',
		'Slug' => 'Varchar(255)',
		'FromPrice' => 'Currency',
		'Status' => 'Enum(array("Active", "Disabled", "Coming soon"))',
		'Featured' => 'Boolean',
		'TripAdvisorReviewId' => 'Varchar(255)',
		'ReviewScore' => 'Decimal',
		'PmsProvider' => 'Enum(array("Not selected", "TheBookingButton", "RMS", "SynXis", "Seekom", "Link out"))',
		'PmsPropertyId' => 'Int',
		'PmsSubPropertyId' => 'Int',
		'PmsBookingLink' => 'Varchar(255)',
		'PlaceRadius' => 'Int',
		'SortOrder' => 'Int',
	];

	private static $has_one = [
		'PropertyLogo' => Imgix::class,
		'AlternativePropertyLogo' => Imgix::class,
	];

	private static $has_many = [
		'Rooms' => Room::class,
		'Restaurants' => Restaurant::class,
		'EventRooms' => EventRoom::class,
		'FeatureGroups' => FeatureGroup::class,
		'Galleries' => Gallery::class,
		'Reviews' => Review::class,
	];

	private static $belongs_many_many = [
		'PropertiesPages' => PropertiesPage::class,
		'Places' => Place::class,
		'Offers' => Offer::class,
	];

	private static $owns = [
		'PropertyLogo',
		'AlternativePropertyLogo',
	];

	private static $default_sort = 'SortOrder';

	private static $indexes = [
		'SlugIndex' => [
			'type' => 'unique',
			'columns' => ['Slug'],
		],
	];

	private static $translate = [
		'Title',
		'Name',
	];

	public static function getActiveProperties(): DataList
	{
		return Property::get()->filter(['Status' => 'Active']);
	}

	public function Offers()
	{
		return $this->getManyManyComponents('Offers')->sort('OfferSortOrder');
	}

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$fields->insertBefore('Name', HeaderField::create('PropertyDetails', 'Property Details'));
		$fields->insertBefore('TripAdvisorReviewId', HeaderField::create('ExternalReviews', 'External Reviews'));
		$fields->insertBefore('PropertyLogo', HeaderField::create('Logo', 'Logo'));
		$fields->removeByName('Title');
		$fields->removeByName('PlaceRadius');
		$fields->removeByName('PropertiesPages');
		$fields->removeByName('Places');
		$fields->removeByName('SortOrder');

		$roomsConfig = GridFieldConfig_RelationEditor::create(20);
		$roomsConfig->addComponent(new GridFieldSortableRows('SortOrder'));
		$roomsGrid = GridField::create('Rooms', 'Rooms', $this->Rooms(), $roomsConfig);
		$fields->addFieldToTab('Root.Rooms', $roomsGrid);

		$offersConfig = GridFieldConfig_RelationEditor::create(20);
		$offersConfig->addComponent(new GridFieldSortableRows('OfferSortOrder'));
		$offersGrid = GridField::create('Offers', 'Offers', $this->Offers(), $offersConfig);
		$fields->addFieldToTab('Root.Offers', $offersGrid);

		$this->hidePmsSettings($fields);

		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function onBeforeWrite() {
		$this->Slug = Slug::generateSlug($this->Name, $this->Slug);
		$this->Title = $this->Name;
		Gallery::createDefaultGalleries($this);
		FeatureGroup::createDefaultFeatureGroups($this);
		parent::onBeforeWrite();
	}

	public function providePermissions()
	{
		$permisions = [
			'MANAGE_PROPERTY_ALL' => [
				'name' => 'Manage all properties',
				'category' => 'Properties',
				'sort' => -100,
			],
		];
		foreach (Property::get() as $key => $property) {
			$permisions['MANAGE_PROPERTY_' . $property->ID] = [
				'name' => 'Manage ' . $property->Name,
				'category' => 'Properties',
				'sort' => $key + 1,
			];
		}
		$this->extend('updatePermisions', $permisions);
		return $permisions;
	}

	public function canView($member = null)
	{
		if (!$member) {
			if (!Security::getCurrentUser()) {
				return $this->Status === 'Disabled' ? false : true;
			}
		}
		return Permission::check(['MANAGE_PROPERTY_ALL', 'MANAGE_PROPERTY_' . $this->ID]);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_PROPERTY_ALL', 'MANAGE_PROPERTY_' . $this->ID]);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_PROPERTY_ALL', 'MANAGE_PROPERTY_' . $this->ID]);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_PROPERTY_ALL', 'MANAGE_PROPERTY_' . $this->ID]);
	}

	public function Link()
	{
		$link = new ArrayList(['/']);
		$this->extend('updatePropertyLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function RoomsLink()
	{
		$link = new ArrayList([$this->Link(), Environment::getEnv('ROOMS_SLUG')]);
		$this->extend('updateRoomsLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function OffersLink()
	{
		$link = new ArrayList([$this->Link(), Environment::getEnv('OFFERS_SLUG')]);
		$this->extend('updateOffersLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function EventRoomsLink()
	{
		$link = new ArrayList([$this->Link(), Environment::getEnv('EVENT_ROOMS_SLUG')]);
		$this->extend('updateEventRoomsLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function RestaurantLink()
	{
		$link = new ArrayList([$this->Link(), Environment::getEnv('RESTAURANTS_SLUG')]);
		$this->extend('updateRestaurantLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function CMSEditLink()
	{
		$admin = Injector::inst()->get(PropertyAdmin::class);
		return Controller::join_links([
			$admin->Link('property'),
			'EditForm',
			'field',
			'property',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink()
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public function getFilteredOffers()
	{
		$offers = new ArrayList([]);
		foreach ($this->Offers() as $offer) {
			if ($offer->isValid()) {
				$offers->add($offer);
			}
		}
		return $offers;
	}

	public function getFilteredRooms()
	{
		$rooms = new ArrayList();
		foreach ($this->Rooms() as $room) {
			if ($room->Status === 'Active') {
				$rooms->push($room);
			}
		}
		return $rooms;
	}

	public function AbsoluteLink()
	{
		return Director::absoluteURL($this->Link());
	}

	public function BuildBookingLink()
	{
		$link = '';
		switch ($this->PmsProvider) {
			case 'SynXis':
				$link = "https://be.synxis.com/?locale=en-GB&Hotel={$this->PmsPropertyId}";
		}
		$this->extend('updateBuildBookingLink', $link);
		return $link;
	}

	protected function onAfterWrite()
	{
		$this->addPlacesWithinRadius();
		parent::onAfterWrite();
	}

	private function hidePmsSettings(FieldList $fields)
	{
		$fields->insertBefore('PmsProvider', HeaderField::create('PMSDetails', 'PMS Details'));
		switch ($this->PmsProvider) {
			case 'Not selected':
				$fields->removeByName('PmsPropertyId');
				$fields->removeByName('PmsSubPropertyId');
				$fields->removeByName('PmsBookingLink');
				break;
			case 'Link out':
			case 'TheBookingButton':
				$fields->removeByName('PmsPropertyId');
				$fields->removeByName('PmsSubPropertyId');
				break;
			case 'RMS':
				$fields->removeByName('PmsBookingLink');
				break;
			case 'Seekom':
			case 'SynXis':
				$fields->removeByName('PmsSubPropertyId');
				$fields->removeByName('PmsBookingLink');
				break;
		}
	}

	private function addPlacesWithinRadius()
	{
		if ($this->Latitude && $this->Longitude) {
			$propertyCoordinates = new LatLong($this->Latitude, $this->Longitude);
			$property = Property::get()->byID($this->ID);
			$places = Place::get();
			foreach ($places as $place) {
				$property->Places()->remove($place);
				if ($place->Latitude && $place->Longitude) {
					try {
						$placeCoordinates = new LatLong($place->Latitude, $place->Longitude);
						$distance = new Calculator($placeCoordinates, $propertyCoordinates);
						$radius = $this->PlaceRadius ? $this->PlaceRadius : 50;
						if ($distance->get()->asKilometres() <= $radius) {
							$property->Places()->add($place);
						}
					} catch (\Exception $e) {
						Injector::inst()->get(LoggerInterface::class)->error($e->getMessage());
					}
				}
			}
		}
	}

}
