<?php

namespace Fastrack\Property;

use Fastrack\Gallery\Gallery;
use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\ORM\ArrayList;
use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;

/**
 * @property String PmsRoomId
 * @property string Name
 * @property string Slug
 * @property int NumberOfBathrooms
 * @property int NumberOfRooms
 * @property int NumberOfGuests
 * @property int SortOrder
 * @property string Excerpt
 * @property string Content
 * @method Imgix Image()
 */
class Room extends DataObject implements PermissionProvider
{
	private static $table_name = 'room';

	private static $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar(255)',
		'Slug' => 'Varchar(255)',
		'Status' => 'Enum(array("Active", "Disabled", "Coming soon"))',
		'NumberOfRooms' => 'Int',
		'BathroomType' => 'Enum(array("Not selected", "Shared", "Private", "Ensuite"))',
		'NumberOfBathrooms' => 'Int',
		'NumberOfBeds' => 'Int',
		'NumberOfGuests' => 'Int',
		'MinimumNumberOfGuests' => 'Int',
		'PetsAllowed' => 'Boolean',
		'SmokingAllowed' => 'Boolean',
		'FromPrice' => 'Currency',
		'PmsRoomId' => 'Varchar(255)',
		'PmsRoomCode' => 'Varchar(255)',
		'BookingLink' => 'Varchar(255)',
		'Featured' => 'Boolean',
		'SortOrder' => 'Int',
	];

	private static $has_one = [
		'Property' => Property::class,
		'RoomType' => RoomType::class,
	];

	private static $has_many = [
		'FeatureGroups' => FeatureGroup::class,
		'Galleries' => Gallery::class,
		'BeddingConfigurations' => BeddingConfiguration::class,
	];

	private static $belongs_to = [
		Property::class,
	];

	private static $default_sort = 'SortOrder';

	private static $indexes = [
		'Slug' => true,
		'SortOrder' => true,
	];

	private static $translate = [
		'Title',
		'Name',
	];

	/**
	 * @config
	 */
	private static array $hidden_fields = [];

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$fields->removeByName('Title');
		$fields->removeByName('SortOrder');
		$fields->removeByName('PropertyID');
		$fields->dataFieldByName('PmsRoomId')->setTitle('PMS Rooom ID');
		ModelUtilities::hideFields($this, $fields);
		ModelUtilities::hideConfigFields(self::class, $fields);
		return $fields;
	}

	public function onBeforeWrite() {
		$this->Title = $this->Name;
		$this->Slug = Slug::generateSlug($this->Name, $this->Slug);
		Gallery::createDefaultGalleries($this);
		FeatureGroup::createDefaultFeatureGroups($this);
		parent::onBeforeWrite();
	}

	public function Link()
	{
		$link = new ArrayList([$this->Property()->RoomsLink(), $this->Slug]);
		$this->extend('updateLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function AbsoluteLink()
	{
		return Director::absoluteURL($this->Link());
	}

	public function CMSEditLink()
	{
		return Controller::join_links([
			$this->Property()->CMSEditLink(),
			'ItemEditForm',
			'field',
			'Rooms',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink()
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}


	public function BuildBookingLink()
	{
		$link = '';
		switch ($this->Property()->PmsProvider) {
			case 'SynXis':
				$link = "{$this->Property()->BuildBookingLink()}&room={$this->PmsRoomCode}";
		}
		$this->extend('updateBuildBookingLink', $link);
		return $link;
	}

	public function getDefaultBookingLink()
	{
		return $this->BookingLink ? $this->BookingLink : $this->Property()->PmsBookingLink;
	}

	public function providePermissions()
	{
		$permisions = [
			'MANAGE_ROOMS' => [
				'name' => 'Manage rooms',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermisions', $permisions);
		return $permisions;
	}

	public function canView($member = null)
	{
		if (!$member) {
			if (!Security::getCurrentUser()) {
				return $this->Status === 'Disabled' ? false : true;
			}
		}
		return Permission::check(['MANAGE_ROOMS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_ROOMS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_ROOMS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_ROOMS']);
	}

	public function useInheritedWorkflow()
	{
		return true;
	}

	public function workflowParent()
	{
		return $this->Property();
	}
}
